#= Simple Wassr Client
#Authors::		Takuya Mannami
#Version::		0.1.3 (2009/11/25)
#License::		MIT License

# -*- coding: utf-8 -*-

require 'net/http'
require 'cgi'
require 'rss'
require 'json'

module WWW
  class Wassr
    WASSR = "api.wassr.jp"
    WASSR_RUBY = "Wassruby"
    WASSR_API_URL = { 
      :UPDATE            => "/statuses/update.json?status=",
      :PUBLIC_TIMELINE   => "/statuses/public_timeline.json",
      :USER_TIMELINE     => "/statuses/user_timeline.json?id=",
      :USER_SHOW         => "/statuses/show.json?id=",
      :NEW_CHANNEL       => "/channel/list.rss",
      :CHANNEL_CHECK     => "/channel/exists.json?name_en=",
      :USER_CHANNEL_LIST => "/channel_user/user_list.json?login_id=",
      :SL_TIMELINE       => "/statuses/sl_timeline.json",
      :TODO_UPDATE       => "/todo/add.json?body=",
      :TODO_LIST         => "/todo/list.json?page=1&done_fg=0",
    }

    def initialize(login_id, password)
      @login_id = login_id
      @password = password
      @http = Net::HTTP.new(WASSR, 80)
    end

    class << self
      def start(login_id, password, &block)
        block.call(WWW::Wassr.new(login_id, password))
      end
      
      def get_body(api_add, id="")
        Net::HTTP.new(WASSR, 80).get(WASSR_API_URL[api_add] << id).body
      end

      def get_res_body_json(api_add, id="")
        JSON.parse(get_body(api_add, id))
      end

      def get_res_body_rss(api_add, id="")
        RSS::Parser.parse(get_body(api_add, id))
      end
      
      def get(key, id=nil)
        case key
        when :public_timeline
          get_res_body_json(:PUBLIC_TIMELINE)
        when :user_timeline 
          get_res_body_json(:USER_TIMELINE, id)     if id != nil
        when :user_show #get new one message of user
          get_res_body_json(:USER_SHOW, id)         if id != nil
        when :new_channel 
          get_res_body_rss(:NEW_CHANNEL)
        when :channel_check 
          get_res_body_json(:CHANNEL_CHECK, id)     if id != nil
        when :sl_timeline
          get_res_body_json(:SL_TIMELINE)
        when :user_channel_list 
          get_res_body_json(:USER_CHANNEL_LIST, id) if id != nil
        when :todo_list 
          get_res_body_json(:TODO_LIST)
        end
      end
    end

    def post!(api_key, val)
      req = Net::HTTP::Post.new(WASSR_API_URL[api_key] << CGI.escape(val) <<
                                "&source=" << WASSR_RUBY)
      req.basic_auth(@login_id, @password)
      res = @http.request(req)
    end
    
    def post(key, val="")
      case key
      when :update
        post!(:UPDATE, val)
      when :todo_update 
        post!(:TODO_UPDATE, val)
      end
    end
  end
end
